// $Id: rpmversion.cc,v 1.3 2001/01/15 23:58:01 kojima Exp $

#ifdef __GNUG__
#pragma implementation "apt-pkg/rpmversion.h"
#endif 

#include <apt-pkg/rpmfactory.h>

#include <alloca.h>
#include <stdlib.h>

#include <rpm/rpmlib.h>
#include <rpm/misc.h>

// VersionCompare (op) - Greater than comparison for versions
// ---------------------------------------------------------------------
/* */
int RPMFactory::versionCompare(const char *A, const char *B)
{
   return versionCompare(A,A + strlen(A),B,B + strlen(B));
}

int RPMFactory::versionCompare(string A,string B)
{
   return versionCompare(A.begin(),A.end(),B.begin(),B.end());
}


/* This fragments the version into E:V-R triples and compares each 
   portion seperately. */
int RPMFactory::versionCompare(const char *A, const char *AEnd, 
				     const char *B, const char *BEnd)
{
    char *bufA;
    char *bufB;
    char *p;
    bool okA, okB;
    char *verA, *verB;
    char *relA, *relB;
    int res;
    
    bufA = (char*)alloca(AEnd-A+4);
    bufB = (char*)alloca(BEnd-B+4);
    
    bufA = strncpy(bufA, A, AEnd-A);
    bufA[AEnd-A] = 0;
    
    bufB = strncpy(bufB, B, BEnd-B);
    bufB[BEnd-B] = 0;
    
    // compare epoch
    p = strchr(bufA, ':');
    okA = (p != NULL);

    p = strchr(bufB, ':');
    okB = (p != NULL);

    if (okA && !okB)
	return 2;
    else if (!okA && okB)
	return -2;
    else if (okA && okB) {
	int epoA, epoB;
	
	p = strchr(bufA, ':');
	*p = 0;
	epoA = atoi(bufA);
	verA = p+1;
	
	p = strchr(bufB, ':');
	*p = 0;
	verB = p+1;
	
	epoB = atoi(bufB);
	if (epoA < epoB)
	    return -1;
	else if (epoA > epoB)
	    return 1;
    } else {
	verA = bufA;
	verB = bufB;
    }
    
    // compare version
    p = strchr(verA, '-');
    if (p) {
	*p = 0;
	relA = p+1;
    } else {
	relA = NULL;
    }
    
    p = strchr(verB, '-');
    if (p) {
	*p = 0;
	relB = p+1;
    } else {
	relB = NULL;
    }

    res = rpmvercmp(verA, verB);
    if (res)
	return res;
    
    // compare release
    if (!relA)
	return 0;

    if (!relB)
	return 0;
    
    res = rpmvercmp(relA, relB);
    
    return res;
}


// CheckDep - Check a single dependency					/*{{{*/
// ---------------------------------------------------------------------
/* This simply preforms the version comparison and switch based on 
   operator. */
bool RPMFactory::checkDep(const char *DepVer,const char *PkgVer,int Op)
{
   int rc;
   int PkgFlags;
   int DepFlags;
   bool invert = false;
   
   DepFlags = 0;
   PkgFlags = RPMSENSE_EQUAL;
   switch (Op & 0x0F)
   {
    case pkgCache::Dep::LessEq:
      DepFlags = RPMSENSE_LESS|RPMSENSE_EQUAL;
      break;

    case pkgCache::Dep::GreaterEq:
      DepFlags = RPMSENSE_GREATER|RPMSENSE_EQUAL;
      break;
      
    case pkgCache::Dep::Less:
      DepFlags = RPMSENSE_LESS;
      break;
      
    case pkgCache::Dep::Greater:
      DepFlags = RPMSENSE_GREATER;
      break;

    case pkgCache::Dep::Equals:
      DepFlags = RPMSENSE_EQUAL;
      break;
      
    case pkgCache::Dep::NotEquals:
      DepFlags = RPMSENSE_EQUAL;
      invert = true;
      break;
      
    default:
      DepFlags = RPMSENSE_ANY; // any version is ok
      break;
   }
   
   rc = rpmRangesOverlap("", PkgVer, PkgFlags, // provide
			 "", DepVer, DepFlags); // request
      
   if (invert)
       return !(rc == 1);
   else
       return (rc == 1);
}

									/*}}}*/
// BaseVersion - Return the upstream version string			/*{{{*/
// ---------------------------------------------------------------------
/* This strips all the debian specific information from the version number */
string RPMFactory::baseVersion(const char *Ver)
{
   // Strip off the bit before the first colon
   const char *I = Ver;
   for (; *I != 0 && *I != ':'; I++);
   if (*I == ':')
      Ver = I + 1;
   
   // Chop off the trailing -
   I = Ver;
   unsigned Last = strlen(Ver);
   for (; *I != 0; I++)
      if (*I == '-')
	 Last = I - Ver;
      
   return string(Ver,Last);
}
									/*}}}*/
