/*
 * $Id: gensrclist.cc,v 1.7 2001/07/12 21:47:33 kojima Exp $
 *
 * $Log: gensrclist.cc,v $
 * Revision 1.7  2001/07/12 21:47:33  kojima
 * ignore duplicated version/diff deps packages
 * new release (cnc51)
 *
 * Revision 1.6  2000/11/06 12:53:49  kojima
 * fixed compile errors for RedHat 6.x (with gcc -Wall -Werror)
 *
 * Revision 1.5  2000/11/01 21:32:28  kojima
 * added manpage
 *
 * Revision 1.3  2000/10/30 02:17:17  kojima
 * fixed bugs in source d/l
 *
 * Revision 1.2  2000/10/29 20:25:10  kojima
 * added support for source download
 *
 * Revision 1.1  2000/10/28 02:23:17  kojima
 * started source support
 *
 */
#include <alloca.h>
#include <ctype.h>
#include <dirent.h>
#include <errno.h>
#include <fcntl.h>
#include <rpm/rpmlib.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <assert.h>

#include <map>
#include <slist>

#include <apt-pkg/error.h>
#include <apt-pkg/tagfile.h>
#include <apt-pkg/rpminit.h>

// from rpmlib
extern  "C" {
   extern int mdfile(const char *fn, unsigned char *digest);
};





int tags[] =  {
   RPMTAG_NAME,
       RPMTAG_EPOCH,
       RPMTAG_VERSION,
       RPMTAG_RELEASE,
       RPMTAG_GROUP,
       RPMTAG_ARCH,
       RPMTAG_PACKAGER,
       RPMTAG_SOURCERPM,
       RPMTAG_SIZE,
       RPMTAG_VENDOR,
       
       RPMTAG_DESCRIPTION, 
       RPMTAG_SUMMARY, 
       
       RPMTAG_REQUIREFLAGS, 
       RPMTAG_REQUIRENAME,
       RPMTAG_REQUIREVERSION
};
int numTags = sizeof(tags) / sizeof(int);


class CachedMD5 {
   char *filename;

   struct FileData {
       string md5;
       time_t timestamp;
   };
   map<string, FileData> md5table;
   
   
public:
   void MD5ForFile(string FileName, time_t timestamp, unsigned char *buf);
   
   CachedMD5(string DirName);
   ~CachedMD5();
};


CachedMD5::CachedMD5(string DirName)
{
    char buf[512];
    FILE *f;
    int i;
    
    sprintf(buf, "%s.md5cache", DirName.c_str());

   
    for (i = 0; buf[i]; i++)
        if (buf[i]=='/')
 	   buf[i]='_';
    filename = strdup(string("/var/cache/apt/gensrclist/"+string(buf)).c_str());

    f = fopen(filename, "r");
    if (!f) {
	return;
    }
    
    while (1) {
	char *p, *pp;
	string file;
	FileData data;

	if (!fgets(buf, 512, f))
	    break;
	
	p = strchr(buf, ' ');
	assert(p);

	file = string(buf, p);
	
	p++;
	pp = strchr(p, ' ');
	assert(p);
	data.md5 = string(p, pp);
	
	pp++;
	data.timestamp = atol(pp);
	
	md5table[file] = data;
    }
    
    fclose(f);
}


CachedMD5::~CachedMD5()
{
    FILE *f;
    
    f = fopen(filename, "w+");
    if (!f) {
	cerr << "gensrclist: could not open file " << filename << " for writing:" <<
	    strerror(errno) << endl;
    } else {
	for (map<string,FileData>::const_iterator iter = md5table.begin();
	     iter != md5table.end(); 
	     iter++) {
	    string file = (*iter).first;
	    const FileData &data = (*iter).second;
	
	    fprintf(f, "%s %s %lu\n", file.c_str(), data.md5.c_str(),
		    data.timestamp);
	}
	fclose(f);
    }
    free(filename);
}


void CachedMD5::MD5ForFile(string FileName, time_t timestamp, 
			   unsigned char *buf)
{
    if (md5table.find(FileName) != md5table.end()
	&& timestamp == md5table[FileName].timestamp) {
	strcpy((char*)buf, md5table[FileName].md5.c_str());
	return;
    }

    FileData data;

    mdfile(FileName.c_str(), buf);
    data.md5 = string((char*)buf);
    data.timestamp = timestamp;

    md5table[FileName] = data;
}




bool readRPMTable(char *file, map<string, slist<char*>* > &table)
{
   FILE *indexf;
   char buf[512];
   string srpm;
   
   indexf = fopen(file, "r");
   if (!indexf) {
      cerr << "gensrclist: could not open file " << file << " for reading: "
	  << strerror(errno) << endl;
      return false;
   }
   
   while (fgets(buf, 512, indexf)) {
      char *f;
      
      buf[strlen(buf)-1] = '\0';
      f = strchr(buf, ' ');
      *f = '\0';
      f++;
      
      srpm = string(buf);
      
      if (table.find(srpm) != table.end()) {
	 slist<char*> *list = table[srpm];
	 
	 list->push_front(strdup(f));
      } else {
	 slist<char*> *list = new slist<char*>;
	 
	 list->push_front(strdup(f));
	 table[srpm] = list;
      }
   }
   
   fclose(indexf);
   
   return true;
}


int main(int argc, char ** argv) 
{
   char buf[300];
   char cwd[200];
   string srpmdir;
   DIR * dir;
   FD_t outfd, fd;
   struct dirent * ent;
   int rc;
   Header h;
   Header sigs;
   struct stat sb;
   int_32 size[1];
   int entry_no, entry_cur;
   CachedMD5 *md5cache;
   map<string, slist<char*>* > rpmTable; // table that maps srpm -> generated rpm
   bool mapi;
   
   putenv("LC_ALL=");
   if (argc < 4 || argc > 5) {
      cerr << "usage: gensrclist <dir> <suffix> <srpm index>" << endl;
      exit(1);
   }
   
   if (argc == 5 && strcmp(argv[4], "--mapi") == 0) // oh yeah, baby!
       mapi = true;
   else
       mapi = false;
   
   if (!readRPMTable(argv[3], rpmTable))
       exit(1);
   
   md5cache = new CachedMD5(string(argv[1])+string(argv[2]));
   
   getcwd(cwd, 200);
   if (*argv[1] != '/') {
      strcpy(buf, cwd);
      strcat(buf, "/");
      strcat(buf, argv[1]);
   } else
       strcpy(buf, argv[1]);
   
   strcat(buf, "/SRPMS.");
   strcat(buf, argv[2]);
   
   srpmdir = "SRPMS." + string(argv[2]);
   
   dir = opendir(buf);
   if (!dir) {
      cerr << "gensrclist: error opening directory " << buf << ":"
	  << strerror(errno) << endl;
      return 1;
   }
   chdir(buf);
   
   sprintf(buf, "%s/srclist.%s", cwd, argv[2]);
   
   unlink(buf);
   
   outfd = fdOpen(buf, O_WRONLY | O_TRUNC | O_CREAT, 0644);
   if (!outfd) {
      cerr << "gensrclist: error creating file" << buf << ":"
	  << strerror(errno);
      return 1;
   }
   
   entry_no = 0;
   
   while (1) {
      ent = readdir(dir);
      if (!ent)
	  break;
      entry_no++;
   }
   
   rewinddir(dir);
   
   errno = 0;
   ent = readdir(dir);
   if (errno) {
      perror("readdir");
      return 1;
   }
   
   entry_cur = 0;
   while (ent) {
      int i = strlen(ent->d_name);
      
      printf("%03i%%\b\b\b\b", 100 * entry_cur / entry_no);
      fflush(stdout);
      
      entry_cur++;
      
      if (i > 4 && strcasecmp(&ent->d_name[i - 4], ".rpm") == 0) {
	 
	 fd = fdOpen(ent->d_name, O_RDONLY, 0666);
	 
	 if (!fd) {
	    cerr << ent->d_name << ":";
	    perror("open");
	    exit(1);
	 }
	 
	 fstat(fdFileno(fd), &sb);
	 size[0] = sb.st_size;
	 
	 rc = rpmReadPackageInfo(fd, &sigs, &h);
	 
	 if (!rc) {
	    Header newHeader;
	    int i;
	    bool foundInIndex;
	    
	    newHeader = headerNew();
	    
	    
	    // the std tags
	    for (i = 0; i < numTags; i++) {
	       int type, count;
	       void *data;
	       int res;
	       
	       res = headerGetEntry(h, tags[i], &type, &data, &count);
	       if (res != 1) {
		  /*
		   printf("warning: tag %i not found on header for %s\n",
		   tags[i], ent->d_name);
		   */
		  continue;
	       }
	       headerAddEntry(newHeader, tags[i], type, data, count);
	    }
	    
	    
	    // our additional tags
	    headerAddEntry(newHeader, CRPMTAG_DIRECTORY, RPM_STRING_TYPE,
			   srpmdir.c_str(), 1);
	    
	    headerAddEntry(newHeader, CRPMTAG_FILENAME, RPM_STRING_TYPE, 
			   ent->d_name, 1);
	    headerAddEntry(newHeader, CRPMTAG_FILESIZE, RPM_INT32_TYPE,
			   size, 1);
	    
	    {
	       unsigned char md5[34];
	       
	       md5cache->MD5ForFile(ent->d_name, sb.st_mtime, md5);
	       
	       headerAddEntry(newHeader, CRPMTAG_MD5, RPM_STRING_TYPE,
			      md5, 1);
	    }
	    
	    foundInIndex = false;
	    {
	       int count = 0;
	       char **list = NULL;
	       slist<char*> *rpmlist = rpmTable[string(ent->d_name)];
	       
	       if (rpmlist) {
		  list = new char *[rpmlist->size()];
		  
		  foundInIndex = true;
		  
		  for (slist<char*>::const_iterator i = rpmlist->begin();
		       i != rpmlist->end();
		       i++) {
		     list[count++] = *i;
		  }
	       }
	       
	       if (count) {
		  headerAddEntry(newHeader, CRPMTAG_BINARY,
				 RPM_STRING_ARRAY_TYPE, list, count);
	       }
	    }
	    if (foundInIndex || !mapi)
		headerWrite(outfd, newHeader, HEADER_MAGIC_YES);
	    
	    headerFree(newHeader);
	    headerFree(h);
	    rpmFreeSignature(sigs);
	 }
	 fdClose(fd);
      }
      
      errno = 0;
      ent = readdir(dir);
      if (errno) {
	 perror("readdir");
	 return 1;
      }
   } 
   
   closedir(dir);
   fdClose(outfd);
   
   delete md5cache;
   
   return 0;
}
